
	This patch fixes a deadlock in the system call auditing code (#30440).
	It also implements the syslog-return action (#31889)

diff -ur audit/drivers/audit/control.c linux/drivers/audit/control.c
--- audit/drivers/audit/control.c	2003-10-06 16:56:41.000000000 +0200
+++ linux/drivers/audit/control.c	2003-10-06 16:58:31.000000000 +0200
@@ -783,6 +783,7 @@
 	task_lock(task);
 	if (task->audit) {
 		DPRINTF("Cannot attach process %d; auditing already enabled\n", task->pid);
+		task_unlock(task);
 		kfree(pinfo);
 		res = -EBUSY;
 	} else {
@@ -795,8 +796,8 @@
 		/* turn on syscall intercept */
 		task->audit = pinfo;
 		task->ptrace |= PT_AUDITED;
+		task_unlock(task);
 	}
-	task_unlock(task);
 	return res;
 }
 
@@ -824,11 +825,12 @@
 	task_lock(task);
 	if ((pinfo = task->audit) == NULL) {
 		res = -EUNATCH;
+		task_unlock(task);
 	} else {
 		/* turn off system call intercept */
 		task->ptrace &= ~PT_AUDITED;
 		task->audit = NULL;
-		wmb();
+		task_unlock(task);
 
 		/* Free any memory we may have allocated for
 	   	 * argument data, and release VM scratch memory */
@@ -836,7 +838,6 @@
 		audit_release_scratch_vm(pinfo);
 		kfree(pinfo);
 	}
-	task_unlock(task);
 	return res;
 }
 
diff -ur audit/drivers/audit/filter.c linux/drivers/audit/filter.c
--- audit/drivers/audit/filter.c	2003-10-06 16:56:41.000000000 +0200
+++ linux/drivers/audit/filter.c	2003-10-06 16:59:05.000000000 +0200
@@ -603,6 +603,8 @@
 		return AUDIT_LOG;
 	if (f->af_op == AUD_FILT_OP_FALSE)
 		return 0;
+	if (f->af_op == AUD_FILT_OP_RETURN)
+		return f->af_action;
 
 	if (tgt == NULL)
 		return -EINVAL;
